@@ -20,20 +20,21 @@
 Unit Tests for remote procedure calls using queue
 """
 
-import mox
+import sys
+import testtools
 
+import mox
+from oslo.config import cfg
 
 from nova import context
 from nova import db
 from nova import exception
-from nova import flags
 from nova import manager
-from nova.openstack.common import cfg
 from nova import service
 from nova import test
+from nova.tests import utils
 from nova import wsgi
 
-
 test_service_opts = [
     cfg.StrOpt("fake_manager",
                default="nova.tests.test_service.FakeManager",
@@ -46,11 +47,12 @@ test_service_opts = [
                help="Port number to bind test service to"),
     ]
 
-flags.FLAGS.register_opts(test_service_opts)
+CONF = cfg.CONF
+CONF.register_opts(test_service_opts)
 
 
 class FakeManager(manager.Manager):
-    """Fake manager for tests"""
+    """Fake manager for tests."""
     def test_method(self):
         return 'manager'
 
@@ -61,7 +63,7 @@ class ExtendedService(service.Service):
 
 
 class ServiceManagerTestCase(test.TestCase):
-    """Test cases for Services"""
+    """Test cases for Services."""
 
     def test_message_gets_to_manager(self):
         serv = service.Service('test',
@@ -105,92 +107,71 @@ class ServiceFlagsTestCase(test.TestCase):
 
 
 class ServiceTestCase(test.TestCase):
-    """Test cases for Services"""
+    """Test cases for Services."""
 
     def setUp(self):
         super(ServiceTestCase, self).setUp()
-        self.mox.StubOutWithMock(service, 'db')
+        self.host = 'foo'
+        self.binary = 'nova-fake'
+        self.topic = 'fake'
+        self.mox.StubOutWithMock(db, 'service_create')
+        self.mox.StubOutWithMock(db, 'service_get_by_args')
+        self.flags(use_local=True, group='conductor')
 
     def test_create(self):
-        host = 'foo'
-        binary = 'nova-fake'
-        topic = 'fake'
 
         # NOTE(vish): Create was moved out of mox replay to make sure that
         #             the looping calls are created in StartService.
-        app = service.Service.create(host=host, binary=binary, topic=topic)
+        app = service.Service.create(host=self.host, binary=self.binary,
+                topic=self.topic)
 
         self.assert_(app)
 
-    def test_report_state_newly_disconnected(self):
-        host = 'foo'
-        binary = 'bar'
-        topic = 'test'
-        service_create = {'host': host,
-                          'binary': binary,
-                          'topic': topic,
+    def _service_start_mocks(self):
+        service_create = {'host': self.host,
+                          'binary': self.binary,
+                          'topic': self.topic,
+                          'report_count': 0}
+        service_ref = {'host': self.host,
+                          'binary': self.binary,
+                          'topic': self.topic,
                           'report_count': 0,
-                          'availability_zone': 'nova'}
-        service_ref = {'host': host,
-                          'binary': binary,
-                          'topic': topic,
-                          'report_count': 0,
-                          'availability_zone': 'nova',
                           'id': 1}
 
-        service.db.service_get_by_args(mox.IgnoreArg(),
-                                      host,
-                                      binary).AndRaise(exception.NotFound())
-        service.db.service_create(mox.IgnoreArg(),
-                                  service_create).AndReturn(service_ref)
-        service.db.service_get(mox.IgnoreArg(),
-                               mox.IgnoreArg()).AndRaise(Exception())
+        db.service_get_by_args(mox.IgnoreArg(),
+                self.host, self.binary).AndRaise(exception.NotFound())
+        db.service_create(mox.IgnoreArg(),
+                service_create).AndReturn(service_ref)
+        return service_ref
+
+    def test_init_and_start_hooks(self):
+        self.manager_mock = self.mox.CreateMock(FakeManager)
+        self.mox.StubOutWithMock(sys.modules[__name__],
+                'FakeManager', use_mock_anything=True)
+        self.mox.StubOutWithMock(self.manager_mock, 'init_host')
+        self.mox.StubOutWithMock(self.manager_mock, 'pre_start_hook')
+        self.mox.StubOutWithMock(self.manager_mock, 'create_rpc_dispatcher')
+        self.mox.StubOutWithMock(self.manager_mock, 'post_start_hook')
+
+        FakeManager(host=self.host).AndReturn(self.manager_mock)
+
+        # init_host is called before any service record is created
+        self.manager_mock.init_host()
+        self._service_start_mocks()
+        # pre_start_hook is called after service record is created,
+        # but before RPC consumer is created
+        self.manager_mock.pre_start_hook()
+        self.manager_mock.create_rpc_dispatcher(None)
+        # post_start_hook is called after RPC consumer is created.
+        self.manager_mock.post_start_hook()
 
         self.mox.ReplayAll()
-        serv = service.Service(host,
-                               binary,
-                               topic,
+
+        serv = service.Service(self.host,
+                               self.binary,
+                               self.topic,
                                'nova.tests.test_service.FakeManager')
         serv.start()
-        serv.report_state()
-        self.assert_(serv.model_disconnected)
-
-    def test_report_state_newly_connected(self):
-        host = 'foo'
-        binary = 'bar'
-        topic = 'test'
-        service_create = {'host': host,
-                          'binary': binary,
-                          'topic': topic,
-                          'report_count': 0,
-                          'availability_zone': 'nova'}
-        service_ref = {'host': host,
-                          'binary': binary,
-                          'topic': topic,
-                          'report_count': 0,
-                          'availability_zone': 'nova',
-                          'id': 1}
-
-        service.db.service_get_by_args(mox.IgnoreArg(),
-                                      host,
-                                      binary).AndRaise(exception.NotFound())
-        service.db.service_create(mox.IgnoreArg(),
-                                  service_create).AndReturn(service_ref)
-        service.db.service_get(mox.IgnoreArg(),
-                               service_ref['id']).AndReturn(service_ref)
-        service.db.service_update(mox.IgnoreArg(), service_ref['id'],
-                                  mox.ContainsKeyValue('report_count', 1))
-
-        self.mox.ReplayAll()
-        serv = service.Service(host,
-                               binary,
-                               topic,
-                               'nova.tests.test_service.FakeManager')
-        serv.start()
-        serv.model_disconnected = True
-        serv.report_state()
-
-        self.assert_(not serv.model_disconnected)
 
 
 class TestWSGIService(test.TestCase):
@@ -205,6 +186,15 @@ class TestWSGIService(test.TestCase):
         self.assertNotEqual(0, test_service.port)
         test_service.stop()
 
+    @testtools.skipIf(not utils.is_ipv6_supported(), "no ipv6 support")
+    def test_service_random_port_with_ipv6(self):
+        CONF.set_default("test_service_listen", "::1")
+        test_service = service.WSGIService("test_service")
+        test_service.start()
+        self.assertEqual("::1", test_service.host)
+        self.assertNotEqual(0, test_service.port)
+        test_service.stop()
+
 
 class TestLauncher(test.TestCase):
 
@@ -214,7 +204,6 @@ class TestLauncher(test.TestCase):
         self.service = service.WSGIService("test_service")
 
     def test_launch_app(self):
-        launcher = service.Launcher()
-        launcher.launch_server(self.service)
+        service.serve(self.service)
         self.assertNotEquals(0, self.service.port)
-        launcher.stop()
+        service._launcher.stop()
