@@ -1,5 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
+# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
 # Copyright (c) 2012 VMware, Inc.
 # Copyright (c) 2011 Citrix Systems, Inc.
 # Copyright 2011 OpenStack Foundation
@@ -19,11 +20,16 @@
 The VMware API VM utility module to build SOAP object specs.
 """
 
+import collections
 import copy
 
 from nova import exception
+from nova.openstack.common.gettextutils import _
+from nova.openstack.common import log as logging
 from nova.virt.vmwareapi import vim_util
 
+LOG = logging.getLogger(__name__)
+
 
 def build_datastore_path(datastore_name, path):
     """Build the datastore compliant path."""
@@ -51,6 +57,10 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
     config_spec.name = instance['uuid']
     config_spec.guestId = os_type
 
+    # Allow nested ESX instances to host 64 bit VMs.
+    if os_type == "vmkernel5Guest":
+        config_spec.nestedHVEnabled = "True"
+
     vm_file_info = client_factory.create('ns0:VirtualMachineFileInfo')
     vm_file_info.vmPathName = "[" + data_store_name + "]"
     config_spec.files = vm_file_info
@@ -75,7 +85,7 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
 
     config_spec.deviceChange = device_config_spec
 
-    # add vm-uuid and iface-id.x values for Quantum
+    # add vm-uuid and iface-id.x values for Neutron
     extra_config = []
     opt = client_factory.create('ns0:OptionValue')
     opt.key = "nvp.vm-uuid"
@@ -96,6 +106,14 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
     return config_spec
 
 
+def get_vm_resize_spec(client_factory, instance):
+    """Provides updates for a VM spec."""
+    resize_spec = client_factory.create('ns0:VirtualMachineConfigSpec')
+    resize_spec.numCPUs = int(instance['vcpus'])
+    resize_spec.memoryMB = int(instance['memory_mb'])
+    return resize_spec
+
+
 def create_controller_spec(client_factory, key, adapter_type="lsiLogic"):
     """
     Builds a Config Spec for the LSI or Bus Logic Controller's addition
@@ -222,13 +240,22 @@ def get_vmdk_attach_config_spec(client_factory,
     return config_spec
 
 
-def get_vmdk_detach_config_spec(client_factory, device):
-    """Builds the vmdk detach config spec."""
+def get_cdrom_attach_config_spec(client_factory,
+                                 datastore,
+                                 file_path,
+                                 cdrom_unit_number):
+    """Builds and returns the cdrom attach config spec."""
     config_spec = client_factory.create('ns0:VirtualMachineConfigSpec')
 
     device_config_spec = []
-    virtual_device_config_spec = delete_virtual_disk_spec(client_factory,
-                                                          device)
+    # For IDE devices, there are these two default controllers created in the
+    # VM having keys 200 and 201
+    controller_key = 200
+    virtual_device_config_spec = create_virtual_cdrom_spec(client_factory,
+                                                           datastore,
+                                                           controller_key,
+                                                           file_path,
+                                                           cdrom_unit_number)
 
     device_config_spec.append(virtual_device_config_spec)
 
@@ -236,6 +263,36 @@ def get_vmdk_detach_config_spec(client_factory, device):
     return config_spec
 
 
+def get_vmdk_detach_config_spec(client_factory, device,
+                                destroy_disk=False):
+    """Builds the vmdk detach config spec."""
+    config_spec = client_factory.create('ns0:VirtualMachineConfigSpec')
+
+    device_config_spec = []
+    virtual_device_config_spec = detach_virtual_disk_spec(client_factory,
+                                                          device,
+                                                          destroy_disk)
+
+    device_config_spec.append(virtual_device_config_spec)
+
+    config_spec.deviceChange = device_config_spec
+    return config_spec
+
+
+def get_vm_extra_config_spec(client_factory, extra_opts):
+    """Builds extra spec fields from a dictionary."""
+    config_spec = client_factory.create('ns0:VirtualMachineConfigSpec')
+    # add the key value pairs
+    extra_config = []
+    for key, value in extra_opts.iteritems():
+        opt = client_factory.create('ns0:OptionValue')
+        opt.key = key
+        opt.value = value
+        extra_config.append(opt)
+        config_spec.extraConfig = extra_config
+    return config_spec
+
+
 def get_vmdk_path_and_adapter_type(hardware_devices):
     """Gets the vmdk file path and the storage adapter type."""
     if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
@@ -285,7 +342,7 @@ def get_rdm_disk(hardware_devices, uuid):
         if (device.__class__.__name__ == "VirtualDisk" and
             device.backing.__class__.__name__ ==
                 "VirtualDiskRawDiskMappingVer1BackingInfo" and
-            device.backing.lunUuid == uuid):
+                device.backing.lunUuid == uuid):
             return device
 
 
@@ -318,6 +375,39 @@ def get_rdm_create_spec(client_factory, device, adapter_type="lsiLogic",
     return create_vmdk_spec
 
 
+def create_virtual_cdrom_spec(client_factory,
+                              datastore,
+                              controller_key,
+                              file_path,
+                              cdrom_unit_number):
+    """Builds spec for the creation of a new Virtual CDROM to the VM."""
+    config_spec = client_factory.create(
+        'ns0:VirtualDeviceConfigSpec')
+    config_spec.operation = "add"
+
+    cdrom = client_factory.create('ns0:VirtualCdrom')
+
+    cdrom_device_backing = client_factory.create(
+        'ns0:VirtualCdromIsoBackingInfo')
+    cdrom_device_backing.datastore = datastore
+    cdrom_device_backing.fileName = file_path
+
+    cdrom.backing = cdrom_device_backing
+    cdrom.controllerKey = controller_key
+    cdrom.unitNumber = cdrom_unit_number
+    cdrom.key = -1
+
+    connectable_spec = client_factory.create('ns0:VirtualDeviceConnectInfo')
+    connectable_spec.startConnected = True
+    connectable_spec.allowGuestControl = False
+    connectable_spec.connected = True
+
+    cdrom.connectable = connectable_spec
+
+    config_spec.device = cdrom
+    return config_spec
+
+
 def create_virtual_disk_spec(client_factory, controller_key,
                              disk_type="preallocated",
                              file_path=None,
@@ -381,14 +471,15 @@ def create_virtual_disk_spec(client_factory, controller_key,
     return virtual_device_config
 
 
-def delete_virtual_disk_spec(client_factory, device):
+def detach_virtual_disk_spec(client_factory, device, destroy_disk=False):
     """
-    Builds spec for the deletion of an already existing Virtual Disk from VM.
+    Builds spec for the detach of an already existing Virtual Disk from VM.
     """
     virtual_device_config = client_factory.create(
                             'ns0:VirtualDeviceConfigSpec')
     virtual_device_config.operation = "remove"
-    virtual_device_config.fileOperation = "destroy"
+    if destroy_disk:
+        virtual_device_config.fileOperation = "destroy"
     virtual_device_config.device = device
 
     return virtual_device_config
@@ -400,7 +491,8 @@ def clone_vm_spec(client_factory, location,
     clone_spec = client_factory.create('ns0:VirtualMachineCloneSpec')
     clone_spec.location = location
     clone_spec.powerOn = power_on
-    clone_spec.snapshot = snapshot
+    if snapshot:
+        clone_spec.snapshot = snapshot
     clone_spec.template = template
     return clone_spec
 
@@ -411,7 +503,8 @@ def relocate_vm_spec(client_factory, datastore=None, host=None,
     rel_spec = client_factory.create('ns0:VirtualMachineRelocateSpec')
     rel_spec.datastore = datastore
     rel_spec.diskMoveType = disk_move_type
-    rel_spec.host = host
+    if host:
+        rel_spec.host = host
     return rel_spec
 
 
@@ -489,10 +582,17 @@ def get_vnc_config_spec(client_factory, port, password):
     opt_port = client_factory.create('ns0:OptionValue')
     opt_port.key = "RemoteDisplay.vnc.port"
     opt_port.value = port
-    opt_pass = client_factory.create('ns0:OptionValue')
-    opt_pass.key = "RemoteDisplay.vnc.password"
-    opt_pass.value = password
-    virtual_machine_config_spec.extraConfig = [opt_enabled, opt_port, opt_pass]
+    extras = [opt_enabled, opt_port]
+    if password:
+        LOG.deprecated(_("The password-based access to VNC consoles will be "
+                         "removed in the next release. Please, switch to "
+                         "using the default value (this will disable password "
+                         "protection on the VNC console)."))
+        opt_pass = client_factory.create('ns0:OptionValue')
+        opt_pass.key = "RemoteDisplay.vnc.password"
+        opt_pass.value = password
+        extras.append(opt_pass)
+    virtual_machine_config_spec.extraConfig = extras
     return virtual_machine_config_spec
 
 
@@ -503,23 +603,64 @@ def search_datastore_spec(client_factory, file_name):
     return search_spec
 
 
+def _get_token(results):
+    """Get the token from the property results."""
+    return getattr(results, 'token', None)
+
+
+def _get_reference_for_value(results, value):
+    for object in results.objects:
+        if object.obj.value == value:
+            return object
+
+
+def _get_object_for_value(results, value):
+    for object in results.objects:
+        if object.propSet[0].val == value:
+            return object.obj
+
+
+def _get_object_from_results(session, results, value, func):
+    while results:
+        token = _get_token(results)
+        object = func(results, value)
+        if object:
+            if token:
+                session._call_method(vim_util,
+                                     "cancel_retrieve",
+                                     token)
+            return object
+
+        if token:
+            results = session._call_method(vim_util,
+                                           "continue_to_get_objects",
+                                           token)
+        else:
+            return None
+
+
+def _cancel_retrieve_if_necessary(session, results):
+    token = _get_token(results)
+    if token:
+        results = session._call_method(vim_util,
+                                       "cancel_retrieve",
+                                       token)
+
+
 def get_vm_ref_from_name(session, vm_name):
     """Get reference to the VM with the name specified."""
     vms = session._call_method(vim_util, "get_objects",
                 "VirtualMachine", ["name"])
-    for vm in vms:
-        if vm.propSet[0].val == vm_name:
-            return vm.obj
-    return None
+    return _get_object_from_results(session, vms, vm_name,
+                                    _get_object_for_value)
 
 
 def get_vm_ref_from_uuid(session, instance_uuid):
     """Get reference to the VM with the uuid specified."""
     vms = session._call_method(vim_util, "get_objects",
                 "VirtualMachine", ["name"])
-    for vm in vms:
-        if vm.propSet[0].val == instance_uuid:
-            return vm.obj
+    return _get_object_from_results(session, vms, instance_uuid,
+                                    _get_object_for_value)
 
 
 def get_vm_ref(session, instance):
@@ -541,10 +682,8 @@ def get_host_ref_from_id(session, host_id, property_list=None):
     host_refs = session._call_method(
                     vim_util, "get_objects",
                     "HostSystem", property_list)
-
-    for ref in host_refs:
-        if ref.obj.value == host_id:
-            return ref
+    return _get_object_from_results(session, host_refs, host_id,
+                                    _get_reference_for_value)
 
 
 def get_host_id_from_vm_ref(session, vm_ref):
@@ -605,7 +744,7 @@ def property_from_property_set(property_name, property_set):
     :return: the value of the property.
     '''
 
-    for prop in property_set:
+    for prop in property_set.objects:
         p = _property_from_propSet(prop.propSet, property_name)
         if p is not None:
             return p
@@ -637,6 +776,13 @@ def get_host_name_from_host_ref(host_ref):
         return p.val
 
 
+def get_vm_state_from_name(session, vm_name):
+    vm_ref = get_vm_ref_from_name(session, vm_name)
+    vm_state = session._call_method(vim_util, "get_dynamic_property",
+                vm_ref, "VirtualMachine", "runtime.powerState")
+    return vm_state
+
+
 def get_stats_from_cluster(session, cluster):
     """Get the aggregate resource stats of a cluster."""
     cpu_info = {'vcpus': 0, 'cores': 0, 'vendor': [], 'model': []}
@@ -652,7 +798,7 @@ def get_stats_from_cluster(session, cluster):
             result = session._call_method(vim_util,
                          "get_properties_for_a_collection_of_objects",
                          "HostSystem", host_mors, ["summary.hardware"])
-            for obj in result:
+            for obj in result.objects:
                 hardware_summary = obj.propSet[0].val
                 # Total vcpus is the sum of all pCPUs of individual hosts
                 # The overcommitment ratio is factored in by the scheduler
@@ -679,37 +825,106 @@ def get_cluster_ref_from_name(session, cluster_name):
     """Get reference to the cluster with the name specified."""
     cls = session._call_method(vim_util, "get_objects",
                                "ClusterComputeResource", ["name"])
-    for cluster in cls:
-        if cluster.propSet[0].val == cluster_name:
-            return cluster.obj
-    return None
+    return _get_object_from_results(session, cls, cluster_name,
+                                    _get_object_for_value)
 
 
 def get_host_ref(session, cluster=None):
     """Get reference to a host within the cluster specified."""
     if cluster is None:
-        host_mor = session._call_method(vim_util, "get_objects",
-                                        "HostSystem")[0].obj
+        results = session._call_method(vim_util, "get_objects",
+                                       "HostSystem")
+        _cancel_retrieve_if_necessary(session, results)
+        host_mor = results.objects[0].obj
     else:
         host_ret = session._call_method(vim_util, "get_dynamic_property",
                                         cluster, "ClusterComputeResource",
                                         "host")
-        if host_ret is None:
-            return
-        if not host_ret.ManagedObjectReference:
-            return
+        if not host_ret or not host_ret.ManagedObjectReference:
+            msg = _('No host available on cluster')
+            raise exception.NoValidHost(reason=msg)
         host_mor = host_ret.ManagedObjectReference[0]
 
     return host_mor
 
 
+def propset_dict(propset):
+    """Turn a propset list into a dictionary
+
+    PropSet is an optional attribute on ObjectContent objects
+    that are returned by the VMware API.
+
+    You can read more about these at:
+    http://pubs.vmware.com/vsphere-51/index.jsp
+        #com.vmware.wssdk.apiref.doc/
+            vmodl.query.PropertyCollector.ObjectContent.html
+
+    :param propset: a property "set" from ObjectContent
+    :return: dictionary representing property set
+    """
+    if propset is None:
+        return {}
+
+    #TODO(hartsocks): once support for Python 2.6 is dropped
+    # change to {[(prop.name, prop.val) for prop in propset]}
+    return dict([(prop.name, prop.val) for prop in propset])
+
+
+def _get_datastore_ref_and_name(data_stores, datastore_regex=None):
+    # selects the datastore with the most freespace
+    """Find a usable datastore in a given RetrieveResult object.
+
+    :param data_stores: a RetrieveResult object from vSphere API call
+    :param datastore_regex: an optional regular expression to match names
+    :return: datastore_ref, datastore_name, capacity, freespace
+    """
+    DSRecord = collections.namedtuple(
+        'DSRecord', ['datastore', 'name', 'capacity', 'freespace'])
+
+    # we lean on checks performed in caller methods to validate the
+    # datastore reference is not None. If it is, the caller handles
+    # a None reference as appropriate in its context.
+    found_ds = DSRecord(datastore=None, name=None, capacity=None, freespace=0)
+
+    # datastores is actually a RetrieveResult object from vSphere API call
+    for obj_content in data_stores.objects:
+        # the propset attribute "need not be set" by returning API
+        if not hasattr(obj_content, 'propSet'):
+            continue
+
+        propdict = propset_dict(obj_content.propSet)
+        # Local storage identifier vSphere doesn't support CIFS or
+        # vfat for datastores, therefore filtered
+        ds_type = propdict['summary.type']
+        ds_name = propdict['summary.name']
+        if ((ds_type == 'VMFS' or ds_type == 'NFS') and
+                propdict['summary.accessible']):
+            if datastore_regex is None or datastore_regex.match(ds_name):
+                new_ds = DSRecord(
+                    datastore=obj_content.obj,
+                    name=ds_name,
+                    capacity=propdict['summary.capacity'],
+                    freespace=propdict['summary.freeSpace'])
+                # find the largest freespace to return
+                if new_ds.freespace > found_ds.freespace:
+                    found_ds = new_ds
+
+    #TODO(hartsocks): refactor driver to use DSRecord namedtuple
+    # using DSRecord through out will help keep related information
+    # together and improve readability and organisation of the code.
+    if found_ds.datastore is not None:
+        return (found_ds.datastore, found_ds.name,
+                    found_ds.capacity, found_ds.freespace)
+
+
 def get_datastore_ref_and_name(session, cluster=None, host=None,
                                datastore_regex=None):
     """Get the datastore list and choose the first local storage."""
     if cluster is None and host is None:
         data_stores = session._call_method(vim_util, "get_objects",
                     "Datastore", ["summary.type", "summary.name",
-                                  "summary.capacity", "summary.freeSpace"])
+                                  "summary.capacity", "summary.freeSpace",
+                                  "summary.accessible"])
     else:
         if cluster is not None:
             datastore_ret = session._call_method(
@@ -722,39 +937,199 @@ def get_datastore_ref_and_name(session, cluster=None, host=None,
                                         "get_dynamic_property", host,
                                         "HostSystem", "datastore")
 
-        if datastore_ret is None:
+        if not datastore_ret:
             raise exception.DatastoreNotFound()
         data_store_mors = datastore_ret.ManagedObjectReference
         data_stores = session._call_method(vim_util,
                                 "get_properties_for_a_collection_of_objects",
                                 "Datastore", data_store_mors,
                                 ["summary.type", "summary.name",
-                                 "summary.capacity", "summary.freeSpace"])
-    for elem in data_stores:
-        ds_name = None
-        ds_type = None
-        ds_cap = None
-        ds_free = None
-        for prop in elem.propSet:
-            if prop.name == "summary.type":
-                ds_type = prop.val
-            elif prop.name == "summary.name":
-                ds_name = prop.val
-            elif prop.name == "summary.capacity":
-                ds_cap = prop.val
-            elif prop.name == "summary.freeSpace":
-                ds_free = prop.val
-        # Local storage identifier
-        if ds_type == "VMFS" or ds_type == "NFS":
-            if not datastore_regex or datastore_regex.match(ds_name):
-                return elem.obj, ds_name, ds_cap, ds_free
-
-    if datastore_regex:
-        raise exception.DatastoreNotFound(
+                                 "summary.capacity", "summary.freeSpace",
+                                 "summary.accessible"])
+    while data_stores:
+        token = _get_token(data_stores)
+        results = _get_datastore_ref_and_name(data_stores, datastore_regex)
+        if results:
+            if token:
+                session._call_method(vim_util,
+                                     "cancel_retrieve",
+                                     token)
+            return results
+        if token:
+            data_stores = session._call_method(vim_util,
+                                               "continue_to_get_objects",
+                                               token)
+        else:
+            if datastore_regex:
+                raise exception.DatastoreNotFound(
                 _("Datastore regex %s did not match any datastores")
                 % datastore_regex.pattern)
+            else:
+                raise exception.DatastoreNotFound()
+    raise exception.DatastoreNotFound()
+
+
+def get_vmdk_backed_disk_uuid(hardware_devices, volume_uuid):
+    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
+        hardware_devices = hardware_devices.VirtualDevice
+
+    for device in hardware_devices:
+        if (device.__class__.__name__ == "VirtualDisk" and
+                device.backing.__class__.__name__ ==
+                "VirtualDiskFlatVer2BackingInfo" and
+                volume_uuid in device.backing.fileName):
+            return device.backing.uuid
+
+
+def get_vmdk_backed_disk_device(hardware_devices, uuid):
+    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
+        hardware_devices = hardware_devices.VirtualDevice
+
+    for device in hardware_devices:
+        if (device.__class__.__name__ == "VirtualDisk" and
+                device.backing.__class__.__name__ ==
+                "VirtualDiskFlatVer2BackingInfo" and
+                device.backing.uuid == uuid):
+            return device
+
+
+def get_vmdk_volume_disk(hardware_devices):
+    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
+        hardware_devices = hardware_devices.VirtualDevice
+
+    for device in hardware_devices:
+        if (device.__class__.__name__ == "VirtualDisk"):
+            return device
+
+
+def get_res_pool_ref(session, cluster, node_mo_id):
+    """Get the resource pool."""
+    if cluster is None:
+        # With no cluster named, use the root resource pool.
+        results = session._call_method(vim_util, "get_objects",
+                                       "ResourcePool")
+        _cancel_retrieve_if_necessary(session, results)
+        # The 0th resource pool is always the root resource pool on both ESX
+        # and vCenter.
+        res_pool_ref = results.objects[0].obj
     else:
-        raise exception.DatastoreNotFound()
+        if cluster.value == node_mo_id:
+            # Get the root resource pool of the cluster
+            res_pool_ref = session._call_method(vim_util,
+                                                  "get_dynamic_property",
+                                                  cluster,
+                                                  "ClusterComputeResource",
+                                                  "resourcePool")
+
+    return res_pool_ref
+
+
+def get_all_cluster_mors(session):
+    """Get all the clusters in the vCenter."""
+    try:
+        results = session._call_method(vim_util, "get_objects",
+                                        "ClusterComputeResource", ["name"])
+        _cancel_retrieve_if_necessary(session, results)
+        return results.objects
+
+    except Exception as excep:
+        LOG.warn(_("Failed to get cluster references %s") % excep)
+
+
+def get_all_res_pool_mors(session):
+    """Get all the resource pools in the vCenter."""
+    try:
+        results = session._call_method(vim_util, "get_objects",
+                                             "ResourcePool")
+
+        _cancel_retrieve_if_necessary(session, results)
+        return results.objects
+    except Exception as excep:
+        LOG.warn(_("Failed to get resource pool references " "%s") % excep)
+
+
+def get_dynamic_property_mor(session, mor_ref, attribute):
+    """Get the value of an attribute for a given managed object."""
+    return session._call_method(vim_util, "get_dynamic_property",
+                                mor_ref, mor_ref._type, attribute)
+
+
+def find_entity_mor(entity_list, entity_name):
+    """Returns managed object ref for given cluster or resource pool name."""
+    return [mor for mor in entity_list if (hasattr(mor, 'propSet') and
+                                           mor.propSet[0].val == entity_name)]
+
+
+def get_all_cluster_refs_by_name(session, path_list):
+    """Get reference to the Cluster, ResourcePool with the path specified.
+
+    The path is the display name. This can be the full path as well.
+    The input will have the list of clusters and resource pool names
+    """
+    cls = get_all_cluster_mors(session)
+    if not cls:
+        return
+    res = get_all_res_pool_mors(session)
+    if not res:
+        return
+    path_list = [path.strip() for path in path_list]
+    list_obj = []
+    for entity_path in path_list:
+        # entity_path could be unique cluster and/or resource-pool name
+        res_mor = find_entity_mor(res, entity_path)
+        cls_mor = find_entity_mor(cls, entity_path)
+        cls_mor.extend(res_mor)
+        for mor in cls_mor:
+            list_obj.append((mor.obj, mor.propSet[0].val))
+    return get_dict_mor(session, list_obj)
+
+
+def get_dict_mor(session, list_obj):
+    """The input is a list of objects in the form
+    (manage_object,display_name)
+    The managed object will be in the form
+    { value = "domain-1002", _type = "ClusterComputeResource" }
+
+    Output data format:
+    dict_mors = {
+                  'respool-1001': { 'cluster_mor': clusterMor,
+                                    'res_pool_mor': resourcePoolMor,
+                                    'name': display_name },
+                  'domain-1002': { 'cluster_mor': clusterMor,
+                                    'res_pool_mor': resourcePoolMor,
+                                    'name': display_name },
+                }
+    """
+    dict_mors = {}
+    for obj_ref, path in list_obj:
+        if obj_ref._type == "ResourcePool":
+            # Get owner cluster-ref mor
+            cluster_ref = get_dynamic_property_mor(session, obj_ref, "owner")
+            dict_mors[obj_ref.value] = {'cluster_mor': cluster_ref,
+                                        'res_pool_mor': obj_ref,
+                                        'name': path,
+                                        }
+        else:
+            # Get default resource pool of the cluster
+            res_pool_ref = get_dynamic_property_mor(session,
+                                                    obj_ref, "resourcePool")
+            dict_mors[obj_ref.value] = {'cluster_mor': obj_ref,
+                                        'res_pool_mor': res_pool_ref,
+                                        'name': path,
+                                        }
+    return dict_mors
+
+
+def get_mo_id_from_instance(instance):
+    """Return the managed object ID from the instance.
+
+    The instance['node'] will have the hypervisor_hostname field of the
+    compute node on which the instance exists or will be provisioned.
+    This will be of the form
+    'respool-1001(MyResPoolName)'
+    'domain-1001(MyClusterName)'
+    """
+    return instance['node'].partition('(')[0]
 
 
 def get_vmdk_adapter_type(adapter_type):
