@@ -18,6 +18,7 @@
 #    under the License.
 
 from nova import context
+from nova.openstack.common.gettextutils import _
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
 from nova.virt import firewall
@@ -63,8 +64,10 @@ class Dom0IptablesFirewallDriver(firewall.IptablesFirewallDriver):
 
     def _provider_rules(self):
         """Generate a list of rules from provider for IP4 & IP6.
+
         Note: We could not use the common code from virt.firewall because
-        XS doesn't accept the '-m multiport' option"""
+        XS doesn't accept the '-m multiport' option.
+        """
 
         ctxt = context.get_admin_context()
         ipv4_rules = []
